<?php


/**
 * UserIdentity represents the data needed to identify a user.
 * It contains the authentication method that checks if the provided
 * data can identity the user.
 * A User can be identified with a valid Facebook ID of a 
 * user who has connected his user account with his facebook profile
 * or a given email address and valid password.
 */
class User_Identity extends CUserIdentity
{
        /*
         * Return value for authentication method for 
         * users thate have not been activated
         */      
        const ERROR_NOT_ACTIVATED = 11;        

        
        /** email for identifying a user. It corresponds to the 
         * user's email within the database.
         * If no such email address exists within the database,
         * authentification is not possible.
         * @see authenticate
         */
        private $email;
    
        /* identities facebook ID*/
        private $facebookId = Null;
        
        /* club id*/
        private $club_id = Null;
        
        /**
         * key that will be used to validate a club change of an
         * already logged in user.
         * @var varchar(70)
         */
        private $clubChangeKey = Null;
                
    
        /**
         * Uses the given facebook id to authenticate a user instead
         * of it's username and password.
         * Calling this method before calling authenticate() guaranties
         * that the authentification will be performed only with the 
         * facebook id.
         * @param type $facebookID facebook id corresponding to the identity
         * @see authenticate()
         */        
        public function useFacebookId($facebookID)
        {
            $this->facebookId = $facebookID;
            $this->username = Null;
            $this->password = Null;
            $this->email = Null;
            $this->clubChangeKey = Null;
        }        
        
        /**
         * Uses the given email address to authenticate a user instead
         * of a Facebook ID
         * Calling this method before calling authenticate() guaranties
         * that the authentification will be performed only with the 
         * the stated email address and the password assigned during
         * object creation.
         * @param type $facebookID facebook id corresponding to the identity
         * @see authenticate()
         */        
        public function useEmail($email)
        {
            $this->facebookId = Null;
            $this->username = Null;
            $this->email = $email;
            $this->clubChangeKey = Null;
        } 

        /**
         * Uses the given club id to set the user's session parameter: club_id
         * if the user is not yet activated for the club with id $clubId,
         * the last used club will be used as session parameter club_id or 
         * club_id is not set. 
         * @param unknown $clubId
         */
        public function useClub($clubId)
        {
        	$this->club_id = $clubId;
        }
        
        
        public function useClubChangeKey($key) {
        	
        	$this->clubChangeKey = $key;
        	$this->facebookId = Null;
        	$this->username = Null;
        	$this->email = Null;
        	$this->password = Null;
        	$this->club_id = Null;
        }
        
                
	/**
	 * Authenticates a user either bye a given facebook ID or an 
         * email-address with it's corresponding password.
         * before this method is use, you have to call useEmail($email)
         * or useFacebookId($facebookID).
         * 
	 * @return int Error code
         * @see useFacebookId($facebookID)
         * @see useEmail($email)
	 */
	public function authenticate()
	{
            if(isset($this->username))
            {
                /*
                 * The username will be determined by either a given
                 * facebook ID or an email-address with it's corresponding
                 * password. 
                 */
                return $this->errorCode=self::ERROR_UNKNOWN_IDENTITY;;
            }
            
//             $connection=Yii::app()->db;
            if(isset($this->clubChangeKey)) {
            	$this->authByClubChangeKey();
            }
            else {
            	if(!isset($this->email))
            	{
            		$this->authByFacebook();
            	}
            	else
            	{
            		$this->authByEmail();
            	}	
            }
            
            return $this->errorCode;               
	}

	/**
	 * performs authentication with help of email
	 * @return Error_Code: ERROR_NONE, ERROR_USERNAME_INVALID, ERROR_NOT_ACTIVATED, ERROR_PASSWORD_INVALID
	 */
	private function authByEmail() {
		
		$connection=Yii::app()->db;
		$querryCommand = $connection->createCommand();
		$querryCommand->setFrom('tbl_user');
		$querryCommand->setWhere("email='" . $this->email . "'");
		$row = $querryCommand->queryRow(true);
		if($row != false)
		{
			/**
			 * check if the user is already activated
			 */
			$activated = $row['activated'];
			if(!$activated)
			{
				return $this->errorCode=self::ERROR_NOT_ACTIVATED;
			}
		
			$dbPAssword = $row['password'];
			/*
			 * check password
			*/
			if($dbPAssword === crypt($this->password, $dbPAssword))
			{
				$this->username = $row['u_id'];
				$u_id = $this->username;
		
				$hasClub = false;
				if(isset($this->club_id)) {
					$hasClubs = HasClubs::model()->findByAttributes(array('u_id' => $u_id, 'club_id' => $this->club_id));
					if(isset($hasClubs)) {
						if($hasClubs->activated) {
							 
							$date = CTimestamp::getDate(false, false, false);
							$hasClubs->last_used = date('Y-m-d G:i:s', $date[0]);
							if($hasClubs->validate()) {
								$updated = $hasClubs->update();
								if($updated) {
									$this->setState('club_id', $this->club_id);
									$hasClub = true;
								}
							}
							 
						}
					}
				}
		
				if(!$hasClub) {
					$hasClubs = HasClubs::model()->findAllByAttributes(array('u_id' => $row['u_id']), array('order'=>'last_used DESC'));
					if(!empty($hasClubs)) {
						if($hasClubs[0]->activated) {
							$club_id = $hasClubs[0]->club_id;
							$this->setState('club_id', $club_id);
							$hasClub = true;
						}
					}
				}
				 
				if($hasClub) {
					$this->assignClubRole($u_id);
				}
		
				return $this->errorCode=self::ERROR_NONE;
			}
		
			return $this->errorCode=self::ERROR_PASSWORD_INVALID;
		}
		else
		{
			return $this->errorCode=self::ERROR_USERNAME_INVALID;
		}
	}
	
	
	/**
	 * performs authentication with help of facebook
	 * @return Error_Code: ERROR_NONE, ERROR_USERNAME_INVALID, ERROR_NOT_ACTIVATED
	 */
	private function authByFacebook() {
	
		/*
		 * Login with facebook ID
		*/
		if(isset($this->facebookId))
		{
			if($this->facebookId == 0)
			{
				return $this->errorCode=self::ERROR_USERNAME_INVALID;
			}
			
			$connection=Yii::app()->db;
			$querryCommand = $connection->createCommand();
			$querryCommand->setFrom('tbl_facebook');
			$querryCommand->setWhere("f_id='" . $this->facebookId . "'");
			$row = $querryCommand->queryRow(true);
			if($row == true)
			{
				$u_id = $row['u_id'];
		
				/**
				 * check if user is activated
				 */
				$querryCommand = $connection->createCommand();
				$querryCommand->setFrom('tbl_user');
				$querryCommand->setWhere("u_id='" . $u_id . "'");
				$row = $querryCommand->queryRow(true);
				if($row != false)
				{
					$activated = $row['activated'];
					if($activated)
					{
						$this->username = $u_id;
		
						$hasClub = false;
						if(isset($this->club_id)) {
							$hasClubs = HasClubs::model()->findByAttributes(array('u_id' => $u_id, 'club_id' => $this->club_id));
							if(isset($hasClubs)) {
								if($hasClubs->activated) {
		
									$date = CTimestamp::getDate(false, false, false);
									$hasClubs->last_used = date('Y-m-d G:i:s', $date[0]);
									if($hasClubs->validate()) {
										$updated = $hasClubs->update();
										if($updated) {
											$this->setState('club_id', $this->club_id);
											$hasClub = true;
										}
									}
									 
								}
							}
						}
		
						if(!$hasClub) {
							$hasClubs = HasClubs::model()->findAllByAttributes(array('u_id' => $u_id), array('order'=>'last_used DESC'));
							if(!empty($hasClubs)) {
								if($hasClubs[0]->activated) {
									$club_id = $hasClubs[0]->club_id;
									$this->setState('club_id', $club_id);
									$hasClub = true;
								}
							}
						}
		
						if($hasClub) {
							$this->assignClubRole($u_id);
						}
		
						return $this->errorCode=self::ERROR_NONE;
					}
					else {
						return $this->errorCode=  self::ERROR_NOT_ACTIVATED;
					}
				}
		
				return $this->errorCode=self::ERROR_USERNAME_INVALID;
			}
			else
			{
				return $this->errorCode=self::ERROR_USERNAME_INVALID;
			}
		}
		else
		{
			return $this->errorCode=self::ERROR_USERNAME_INVALID;
		}
	}
	
	
	/**
	 * performs authentication with help of club change key
	 * @return Error_Code: ERROR_NONE, ERROR_USERNAME_INVALID, ERROR_NOT_ACTIVATED, ERROR_PASSWORD_INVALID
	 */
	private function authByClubChangeKey() {
		
		if(isset($this->clubChangeKey)) {
			
			$key = ClubChangeKey::model()->findByPk($this->clubChangeKey);
			if($key && isset($key->club_id) && isset($key->u_id)) {
				
				$user = User::model()->findByPk($key->u_id);
				if($user && isset($user->activated) && $user->activated==true) {
					
					$club = Club::model()->findByPk($key->club_id);
					if(isset($club)) {

						$hasClubs = HasClubs::model()->findByPk(array('club_id' => $key->club_id, 'u_id' => $key->u_id));
						if(isset($hasClubs) && isset($hasClubs->activated) && $hasClubs->activated==true) {
							
							$connection=Yii::app()->db;
	                    	$transaction=$connection->beginTransaction();
	                    	try
	                    	{
	                    		$this->username = $key->u_id;
	                    		$this->club_id = $key->club_id;
	                    		$u_id = $this->username;
	                    			
	                    		$hasClub = false;
	                    			
	                    		$date = CTimestamp::getDate(false, false, false);
	                    		$hasClubs->last_used = date('Y-m-d G:i:s', $date[0]);
	                    		if($hasClubs->validate()) {
	                    			$updated = $hasClubs->update();
	                    			if($updated) {
	                    				$this->setState('club_id', $this->club_id);
	                    				$hasClub = true;
	                    			}
	                    		}
	                    			
	                    		if(!$hasClub) {			// TODO: this one correct ? 
	                    								// additional redirect needed ??
	                    			$hasClubs = HasClubs::model()->findAllByAttributes(array('u_id' => $key->u_id), array('order'=>'last_used DESC'));
	                    			if(!empty($hasClubs)) {
	                    				if($hasClubs[0]->activated) {
	                    					$club_id = $hasClubs[0]->club_id;
	                    					$this->setState('club_id', $club_id);
	                    					$hasClub = true;
	                    				}
	                    			}
	                    		}
	                    		
	                    		if($hasClub) {
	                    			$this->assignClubRole($u_id);
	                    		}
	                    		else {
	                    			throw new CDbException('cannot assign club to user');
	                    		}
	                    			
	                    		// all checks complete --> delte club-change-key
	                    		$del = $key->delete();	                    		
	                    		
	                    		$transaction->commit();
	                    	}
	                    	catch(Exception $e) // an exception is raised if a query fails
	                    	{
	                    		
	                    		$transaction->rollback();
	                    		return $this->errorCode=self::ERROR_USERNAME_INVALID;
	                    	}
							
	                    	
							return $this->errorCode=self::ERROR_NONE;
						}
						else {
							return $this->errorCode=self::ERROR_NOT_ACTIVATED;
						}
					}
					else {
						return $this->errorCode=self::ERROR_USERNAME_INVALID;
					}
				}
				else {
					return $this->errorCode=self::ERROR_USERNAME_INVALID;
				}
			}
			else {
				return $this->errorCode=self::ERROR_USERNAME_INVALID;
			}
		}
		
		return $this->errorCode=self::ERROR_USERNAME_INVALID;
	}
	
	/**
	 * assigns a ClubRole to a given user.
	 * The user's role will be take from DB 
	 * and all previously assigned roles will be deleted.
	 * if no role could be found within the DB, the default 
	 * role will be assigned.
	 *  
	 * @param integer $u_id
	 */
	public function assignClubRole($u_id) {
		
// 		User_Identity::removeRolePrivilidges($u_id);		// TODO: add again ? 
		
		// TODO: assert(isset($this->getState('club_id')));
		$auth=Yii::app()->authManager;
		
		$cId = $this->getState('club_id');
		$currClub = HasClubs::model()->findByPk(array('u_id' => $u_id, 'club_id' => $cId));
		if(isset($currClub)) {			//TODO; check if club is activated ???
			/*
			 * assign new roles if available
			* (currently we can only have one role)
			*/
			if(isset($currClub->role)) {
				echo "";
				$bizRule = 'return Controller::getClubId(Null, $_GET["clubUrl"]) == $data["club_id"];';
				if($auth->assign($currClub->role, $u_id, $bizRule, array('club_id' => $cId)) ) {
					Yii::app()->authManager->save(); //saves the above declaration
				}
			}
		}
	}
	
	/**
	 * 
	 */
	public static function removeRolePrivilidges($u_id) {
		$auth=Yii::app()->authManager;
		
		// TODO: get all CAuthAssignment from DB
		// there can only be one for each club.
		// --> delete the one with the desired club.
		//
		// TODO: system assumes, that there can be only one AuthItem
		// for a given user, i.e a user can not be admin in two different clubs.
		// 
		// 
		
		/*
		 * delete all assigned roles
		*/
		$assigned_roles = Yii::app()->authManager->getRoles($u_id);
		if(!empty($assigned_roles)) {
			foreach($assigned_roles as $n=>$role) {
				if($auth->revoke($n, $u_id)) //remove each assigned role for this user
					Yii::app()->authManager->save();
			}
		}
	}
	
        /**
         * Receive the id of a user. 
         * It's email field has to be set and a corresponding email entry
         * within the database is needed.
         * 
         * @return int the userID or -1 if the user's email field is not set or
         * there is no user with the given email address within the database.
         */
        /*
        public function getUserId()
        {
            if($this->email != Null)
            {
                $connection=Yii::app()->db;
                $querryCommand = $connection->createCommand();
                $querryCommand->setFrom('tbl_user');
                $querryCommand->setWhere("email='" . $this->email . "'");
                $row = $querryCommand->queryRow(true);
                if($row != false)
                {
                    return $row['u_id'];
                }
            }
            
            return -1;
        }
        */
        
        /**
        * Generate a random salt in the crypt(3) standard Blowfish format.
        *
        * @param int $cost Cost parameter from 4 to 31.
        *
        * @throws Exception on invalid cost parameter.
        * @return string A Blowfish hash salt for use in PHP's crypt()
        */
       public static function blowfishSalt($cost = 13)
       {
           $salt='no real salt here';
           return $salt;
       }
       
       /**
        * creates a random password of length $length.
        * @param number $length default length = 4
        * @return string
        */
       public static function createRandomPassword($length = 4)
       {       		
       		$chars = array_merge(range(0,9), range('a','z'), range('A','Z'));
       		shuffle($chars);
       		$password = implode(array_slice($chars, 0, $length));
       		return $password;
       }
}